/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {errorDialog} from "../../../helpers/dialogHelper";


import {createAndRefreshAssetUrlsAndObjects, freeAssetUrlsAndObjects} from "../../../helpers/assetHelper";
import Spinner from "../../helpers/spinner";
import PanelGrid from "../../panelSystem/panelGrid/panelGrid";
import Panel from "../../panelSystem/panel/panel";
import TabContentHost from "../../panelSystem/panel/tabContentHost";
import EditorSettingsDialog from '../../editorSettingsDialog/editorSettingsDialog'
import {
  loadEditCustomProjectSite,
  resetCustomProjectTestResults,
  setAddCustomProjectAssetFilesModalFilterText,
  setAddCustomProjectAssetFilesModalUploadPercentage,
  setBotLeftPanelViewSize,
  setBotRightPanelViewSize,
  setBottomLeftSelectedTabIndex,
  setBottomRightSelectedTabIndex,
  setCustomProjectCodeEditorSettingsDialogDisplayed,
  setCustomProjectHDividerPosOffset,
  setCustomProjectVDividerPosOffset,
  setIsAddCustomProjectAssetFilesModalDisplayed,
  setIsChoosePLangDialogDisplayed,
  setSelectedConsoleViewTest,
  setTopLeftPanelViewSize,
  setTopLeftSelectedTabIndex,
  setTopRightPanelViewSize,
  setTopRightSelectedTabIndex,
} from "../../../state/actions/editCustomProjectSite/editCustomProjectActions";

import {
  setCustomProjectDescriptionAssets,
  setDescriptionContent
} from "../../../state/actions/editCustomProjectSite/subSet/customProjectDescriptionActions";
import {parseId} from "../../../helpers/convertersAndTransformers";
import {RouteComponentProps, withRouter} from "react-router";
//---top right panel
import PreviewPanelView from '../exerciseEditorSite/previewPanel/previewPanelView'
import ExerciseSyntaxHelpPanelView from '../exerciseEditorSite/exerciseSyntaxHelpPanel/exerciseSyntaxHelpPanelView'
import ExerciseSyntaxHelpPanelHeaderBar
  from '../exerciseEditorSite/exerciseSyntaxHelpPanel/exerciseSyntaxHelpPanelHeaderBar'

import PropertiesPanelView from './propertiesPanel/propertiesPanelView'
//--- top left panel
import EditorPanelView from './editorPanel/editorPanelView'
import EditorPanelHeaderBar from './editorPanel/editorPanelHeaderBar'

import DescriptionEditorPanelView from '../exerciseEditorSite/descriptionEditorPanel/descriptionEditorPanelView'
import DescriptionEditorHeaderBar from '../exerciseEditorSite/descriptionEditorPanel/descriptionEditorHeaderBar'
//--- bottom left panel
import CustomTestsPanel from '../doExerciseSite/customTestsPanel/customTestsPanelView'
import CustomTestPanelHeaderBar from '../doExerciseSite/customTestsPanel/customTestsPanelHeaderBar'

import AssetsPanelView from '../exerciseEditorSite/assetsPanel/assetsPanelView'
import AssetsPanelHeaderBar from '../exerciseEditorSite/assetsPanel/assetsPanelHeaderBar'
//--- bottom right panel
import ConsolePanelView from '../doExerciseSite/consolePanel/consolePanelView'
import ConsolePanelHeaderBar from '../doExerciseSite/consolePanel/consolePanelHeaderBar'
import {RunnableCustomProjectTestFrontendExclusive} from "../../../types/customProjects";


import ChangeTestDialog from './dialogs/changeTestDialog/changeTestDialog'
import {
  openCreateCustomProjectTestDialog,
  openEditCustomProjectTestDialog,
  resetChangeCustomProjectTestDialog
} from "../../../state/actions/editCustomProjectSite/dialogs/changeCustomProjectTestDialog/changeCustomProjectTestDialogActions";
import {LoginTimeoutHelper} from "../../../helpers/loginTimeoutHelper";
import Logger from "../../../helpers/logger";
import {deleteCustomTestCustomProjectAsync} from "../../../state/actions/editCustomProjectSite/subSet/customProjectTestActions";
import {
  _getSingleCustomProjectDescriptionAssetFileAsync,
  attachExitingFileToCustomProjectDescriptionAsync,
  detachFileFromCustomProjectDescriptionAsync,
  runCustomProjectTestAsync,
  saveCustomProjectSolution
} from "../../../state/actions/editCustomProjectSite/editCustomProjectCrudActions";
import {RunTestCommandCustomProjectForBackend} from "../../../types/testsAndTestProtocol";
import {getI18n} from "../../../../i18n/i18nRoot";
import {AceEditorsStorageHelper} from '../../codeEditors/aceEditors/aceEditorsStorageHelper'
import {ErrorHelper} from '../../../helpers/errorHelper'
import ZoomPreviewPanelActions from '../exerciseEditorSite/previewPanel/zoomPreviewPanelActions'
import ChoosePLangDialog from './dialogs/choosPLangModal/choosePLangModal'
import ManageAssetFilesModal from '../exerciseEditorSite/addAssetFilesModal/addAssetFilesModal'
import {FilePreviewFromBackend} from '../../../types/exerciseEditor'
import axios, {CancelTokenSource} from 'axios'
import {getPossibleAllUserFilesAsync} from '../../../state/actions/doExerciseSite/doExerciseCrudActions'
import {uploadCustomProjectDescriptionAssetFile} from '../../../communicationLayer/editCustomProjectLayer'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    systemSettings: rootState.globalSystemSettingsState.systemSettings,

    taskDescriptionFontSizeInPx: rootState.globalUiState.taskDescriptionFontSizeInPx,

    customProjectDescription: rootState.editCustomProjectSiteState.customProject.description,
    customProject: rootState.editCustomProjectSiteState.customProject,

    testTypes: rootState.testTypesState.testTypes,
    pLangs: rootState.pLangsState.pLangs,


    editorHasUnsavedChanges: rootState.editCustomProjectSiteState.solutionEditorState.hasUnsavedChanges,
    canCreateCustomProject: rootState.manageCustomProjectsSiteState.customProjectPreviews.length < rootState.globalSystemSettingsState.systemSettings.maxCustomProjectsPerUser || rootState.globalSystemSettingsState.systemSettings.maxCustomProjectsPerUser < 0,

    selectedConsoleViewTest: rootState.editCustomProjectSiteState.selectedConsoleViewTest,

    topRightSelectedTabIndex: rootState.editCustomProjectSiteState.topRightSelectedTabIndex,
    topLeftSelectedTabIndex: rootState.editCustomProjectSiteState.topLeftSelectedTabIndex,
    bottomRightSelectedTabIndex: rootState.editCustomProjectSiteState.bottomRightSelectedTabIndex,
    bottomLeftSelectedTabIndex: rootState.editCustomProjectSiteState.bottomLeftSelectedTabIndex,

    topRightPanelViewSize: rootState.editCustomProjectSiteState.topRightPanelViewSize,
    topLeftPanelViewSize: rootState.editCustomProjectSiteState.topLeftPanelViewSize,
    botRightPanelViewSize: rootState.editCustomProjectSiteState.botRightPanelViewSize,
    botLeftPanelViewSize: rootState.editCustomProjectSiteState.botLeftPanelViewSize,

    vDividerPosOffset: rootState.editCustomProjectSiteState.vDividerPosOffset,
    hDividerPosOffset: rootState.editCustomProjectSiteState.hDividerPosOffset,

    isChangeTestDialogDisplayed: rootState.editCustomProjectSiteState.isChangeTestDialogDisplayed,
    isEditorSettingsDialogDisplayed: rootState.editCustomProjectSiteState.isCustomProjectCodeEditorSettingsDialogDisplayed,

    isChoosePLangDialogDisplayed: rootState.editCustomProjectSiteState.isChoosePLangDialogDisplayed,

    isSomeTestRunning: rootState.editCustomProjectSiteState.customProject.tests.some(p => p.uiIsRunning),

    isLoading: rootState.editCustomProjectSiteState.isLoading
      || rootState.pLangsState.isLoading
      || rootState.testTypesState.isLoading
      || rootState.manageCustomProjectsSiteState.isLoading
      || rootState.globalSystemSettingsState.isLoading,

    isNoDialogDisplayed: !rootState.editCustomProjectSiteState.isCustomProjectCodeEditorSettingsDialogDisplayed
      && !rootState.editCustomProjectSiteState.isChangeTestDialogDisplayed,

    isAddCustomProjectAssetFilesModalDisplayed: rootState.editCustomProjectSiteState.isAddCustomProjectAssetFilesModalDisplayed,
    isAddCustomProjectAssetFilesModalBusy: rootState.editCustomProjectSiteState.isAddCustomProjectAssetFilesModalBusy,
    addCustomProjectAssetFilesModalFilterText: rootState.editCustomProjectSiteState.addCustomProjectAssetFilesModalFilterText,
    addCustomProjectAssetFilesModalUploadPercentage: rootState.editCustomProjectSiteState.addCustomProjectAssetFilesModalUploadPercentage,
    allCustomProjectAssetFilePreviews: rootState.editCustomProjectSiteState.allCustomProjectAssetFilePreviews,

    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setCustomProjectVDividerPosOffset,
                                                                        setCustomProjectHDividerPosOffset,
                                                                        setTopLeftSelectedTabIndex,
                                                                        setTopLeftPanelViewSize,
                                                                        setTopRightSelectedTabIndex,
                                                                        setTopRightPanelViewSize,
                                                                        setBottomLeftSelectedTabIndex,
                                                                        setBotLeftPanelViewSize,
                                                                        setBottomRightSelectedTabIndex,
                                                                        setBotRightPanelViewSize,

                                                                        setCustomProjectCodeEditorSettingsDialogDisplayed,
                                                                        setDescriptionContent,
                                                                        setCustomProjectDescriptionAssets,
                                                                        loadEditCustomProjectSite,

                                                                        resetChangeCustomProjectTestDialog,
                                                                        openCreateCustomProjectTestDialog,
                                                                        openEditCustomProjectTestDialog,
                                                                        setSelectedConsoleViewTest,

                                                                        deleteCustomTestCustomProjectAsync,
                                                                        runCustomProjectTestAsync,
                                                                        saveCustomProjectSolution,
                                                                        resetCustomProjectTestResults,
                                                                        setIsChoosePLangDialogDisplayed,


                                                                        setIsAddCustomProjectAssetFilesModalDisplayed,
                                                                        setAddCustomProjectAssetFilesModalFilterText,
                                                                        setAddCustomProjectAssetFilesModalUploadPercentage,
                                                                        attachExitingFileToCustomProjectDescriptionAsync,
                                                                        detachFileFromCustomProjectDescriptionAsync,
                                                                        getPossibleAllUserFilesAsync,
                                                                        _getSingleCustomProjectDescriptionAssetFileAsync,

                                                                      }, dispatch)


interface MatchedProps {
  /**
   * the id to load or undefined to create a new one
   */
  readonly customProjectId: string | undefined
}

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

const customProjectTaskPrintId = 'custom-project-task-print-id'

// tslint:disable-next-line
let currentFileUploadCancelToken: CancelTokenSource | null = null

class editCustomProjectSite extends React.Component<Props & RouteComponentProps<MatchedProps>, any> {

  async componentWillMount(): Promise<void> {

    // noinspection TsLint
    const customProjectId = parseId(this.props.match.params.customProjectId)

    if (customProjectId !== null) {
      //load

      if (LoginTimeoutHelper.isReLogin && customProjectId === LoginTimeoutHelper.lastKnownCustomProjectId) {
        //do not reload or reset site state...
        //user re logged in he/she want's to continue

        createAndRefreshAssetUrlsAndObjects(this.props.customProject.description.assets)

        return
      } else {
        LoginTimeoutHelper.lastKnownCustomProjectId = customProjectId
      }

      try {
        //reset ace storage
        AceEditorsStorageHelper.removeAll()
        try {
          await this.props.loadEditCustomProjectSite(customProjectId, false)
        } catch (err) {
          ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
          Logger.error(`could not load custom project with id: ${customProjectId}`)
        }
      } catch (err) {

        ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Server error', 'Could not load custom project')
        Logger.error(`could not load custom project with id: ${customProjectId}`)
        return
      }

    } else {
      //create

      if (this.props.customProject.id <= 0) {

        if (LoginTimeoutHelper.isReLogin && this.props.customProject
          && this.props.customProject.id === LoginTimeoutHelper.lastKnownCustomProjectId
        ) {
          //do not reload or reset site state...
          //user re logged in he/she want's to continue
          return
        } else {
          //this would give us an old id because loadEditCustomProjectSite will create a new one...
          //LoginTimeoutHelper.lastKnownExerciseEditorId = this.props.editorExercise.id
        }
      }


      //reset ace storage
      AceEditorsStorageHelper.removeAll()
      try {
        await this.props.loadEditCustomProjectSite(null, true)
      } catch (err) {
        ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
      }

      //ensure the user can create a new project
      if (this.props.canCreateCustomProject === false) {
        errorDialog(getI18n(this.props.langId, 'Error'),
                    getI18n(this.props.langId,
                            'Could not create custom project because you exceeded the max amount of possible custom projects.'
                    ), this.props.langId
        )
        return
      }

      //grab new id
      LoginTimeoutHelper.lastKnownCustomProjectId = customProjectId

      //use has not chose a plang yet
      this.props.setIsChoosePLangDialogDisplayed(true)
    }
  }

  componentWillUnmount(): void {
    freeAssetUrlsAndObjects()
  }

  render(): JSX.Element {
    return (
      <div className="fh">

        {
          this.props.isLoading &&
          <Spinner/>
        }


        <ManageAssetFilesModal
          title={getI18n(this.props.langId, "Manage asset files")}
          isDisplayed={this.props.isAddCustomProjectAssetFilesModalDisplayed}
          isBusy={this.props.isAddCustomProjectAssetFilesModalBusy}
          uploadPercentage={this.props.addCustomProjectAssetFilesModalUploadPercentage}
          uploadInputId={'custom-project-description-upload'}
          filePreviewsListSourcesTooltip={"These are all files from custom projects and custom exercise tests."}

          onUploadFile={async (file) => {

            this.props.setIsAddCustomProjectAssetFilesModalDisplayed(true, true)
            this.props.setAddCustomProjectAssetFilesModalUploadPercentage(0)

            currentFileUploadCancelToken = axios.CancelToken.source()

            try {
              const assetFilePreview = await uploadCustomProjectDescriptionAssetFile(this.props.customProject.id, file,
                                                                                currentFileUploadCancelToken.token,
                                                                                this.props.setAddCustomProjectAssetFilesModalUploadPercentage
              )

              currentFileUploadCancelToken = null

              //reset for download
              //for some reason download shows total of 0, see https://github.com/axios/axios/issues/1591
              this.props.setAddCustomProjectAssetFilesModalUploadPercentage('active')
              Logger.debug('editCustomProjectSite...editCustomProjectSite',
                           `upload finished (new id: ${assetFilePreview.id}), now downloading file content again...`
              )

              //this will download the file ... even if we uploaded it (this is ok, we check that everything went well)
              await this.props._getSingleCustomProjectDescriptionAssetFileAsync(this.props.customProject.id,
                                                                           assetFilePreview.id,
                                                                           // this.props.setAddAssetFilesModalUploadPercentage
              )

              //renew all asset preview list (could have changed) this is easier than to manage all special cases
              //e.g. detach file... was it deleted or just detached??
              // this.props.finishAddAssetFilesModalUpload(assetFilePreview)
              await this.props.getPossibleAllUserFilesAsync()

            } catch (err) {
              Logger.error('could not upload file, err: ' + err.message)
            } finally {
              currentFileUploadCancelToken = null
              this.props.setIsAddCustomProjectAssetFilesModalDisplayed(true, false)
              this.props.setAddCustomProjectAssetFilesModalUploadPercentage(null)
            }

          }}

          onUploadCanceled={
            typeof this.props.addCustomProjectAssetFilesModalUploadPercentage === 'number'
            ? () => {
              if (currentFileUploadCancelToken === null) return
              Logger.debug('editCustomProjectSite...editCustomProjectSite', 'canceled file upload')
              currentFileUploadCancelToken.cancel('canceled')
            }
            : null
          }

          filterText={this.props.addCustomProjectAssetFilesModalFilterText}
          onFilterTextChanged={(text) => {
            this.props.setAddCustomProjectAssetFilesModalFilterText(text)
          }}

          attachedAssetFilePreviews={this.props.customProject.description.assets.map<FilePreviewFromBackend>(p => {
            return {
              id: p.id,
              originalName: p.displayName,
              sizeInBytes: p.content.length,
              mimeType: p.mimeType
            }
          })}

          allAssetFilePreviews={this.props.allCustomProjectAssetFilePreviews}
          onAttachAssetFilePreview={async (assetFilePreview) => {
            this.props.setAddCustomProjectAssetFilesModalUploadPercentage('active')

            //this will download the file so display the progress bar
            //TODO
            try {
              await this.props.attachExitingFileToCustomProjectDescriptionAsync(this.props.customProject.id,
                                                                           assetFilePreview.id,
                                                                           // this.props.setAddAssetFilesModalUploadPercentage
              )
            } catch (err) {
              //not handled here
            } finally {
              this.props.setAddCustomProjectAssetFilesModalUploadPercentage(null)
            }
          }}

          onDetachAssetFilePreview={async (assetFilePreview) => {
            await this.props.detachFileFromCustomProjectDescriptionAsync(this.props.customProject.id, assetFilePreview.id)

            //refresh list
            await this.props.getPossibleAllUserFilesAsync()
          }}

          onClose={() => {
            this.props.setIsAddCustomProjectAssetFilesModalDisplayed(false, false)
          }}

        />




        {
          //only an overlay not a full site dialog...
          this.props.isChoosePLangDialogDisplayed &&
          <ChoosePLangDialog/>
        }

        {
          this.props.isChangeTestDialogDisplayed &&
          <ChangeTestDialog/>
        }

        {
          this.props.isEditorSettingsDialogDisplayed &&
          <EditorSettingsDialog/>
        }

        {
          this.props.isNoDialogDisplayed &&
          <PanelGrid
            width="100%"
            height="100%"

            topRightPanelViewSize={this.props.topRightPanelViewSize}
            topLeftPanelViewSize={this.props.topLeftPanelViewSize}
            botRightPanelViewSize={this.props.botRightPanelViewSize}
            botLeftPanelViewSize={this.props.botLeftPanelViewSize}

            hDividerPosOffset={this.props.hDividerPosOffset}
            vDividerPosOffset={this.props.vDividerPosOffset}
            onHDividerPosOffsetChanged={(Offset: number) => {
              this.props.setCustomProjectHDividerPosOffset(Offset)
            }}
            onVDividerPosOffsetChanged={(Offset: number) => {
              this.props.setCustomProjectVDividerPosOffset(Offset)
            }}

            topLeftPanel={
              <Panel
                applyDefaultPadding={false}
                selectedTabIndex={this.props.topLeftSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setTopLeftSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setTopLeftPanelViewSize}
                panelViewSize={this.props.topLeftPanelViewSize}
              >

                <TabContentHost
                  title={getI18n(this.props.langId, 'Solution editor')}
                  icon={'write'}
                  applyDefaultPadding={false}
                  tabHeaderBar={
                    <EditorPanelHeaderBar/>
                  }
                >
                  <EditorPanelView/>
                </TabContentHost>

                <TabContentHost
                  icon={'font'}
                  title={getI18n(this.props.langId, 'Description editor')}
                  applyDefaultPadding={false}
                  tabHeaderBar={
                    <DescriptionEditorHeaderBar
                      setIsEditorSettingsDialogDisplayed={this.props.setCustomProjectCodeEditorSettingsDialogDisplayed}
                    />
                  }
                >
                  <DescriptionEditorPanelView
                    descriptionContent={this.props.customProjectDescription.content}
                    setContent={(content: string) => {
                      this.props.setDescriptionContent(content)
                    }}
                  />
                </TabContentHost>

              </Panel>
            }

            topRightPanel={
              <Panel
                selectedTabIndex={this.props.topRightSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setTopRightSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setTopRightPanelViewSize}
                panelViewSize={this.props.topRightPanelViewSize}
                additionalActionsRightIcons={
                  this.props.topRightSelectedTabIndex === 1
                  ? <ZoomPreviewPanelActions printId={customProjectTaskPrintId} canPrint={true}/>
                  : undefined
                }
              >

                <TabContentHost
                  title={getI18n(this.props.langId, 'Properties')}
                  icon={'options'}
                >
                  <PropertiesPanelView/>
                </TabContentHost>

                <TabContentHost
                  title={getI18n(this.props.langId, 'Preview')}
                  icon={'unhide'}
                >
                  <PreviewPanelView printId={customProjectTaskPrintId}
                                    descriptionContent={this.props.customProjectDescription.content}
                                    assets={this.props.customProjectDescription.assets}
                                    dummy={false /*not needed because we have tabs that will re render if tab index changed*/}
                                    fontSizeInPx={this.props.taskDescriptionFontSizeInPx}
                  />
                </TabContentHost>

                <TabContentHost
                  title={getI18n(this.props.langId, 'Guide')}
                  icon={'book'}
                  tabHeaderBar={
                    <ExerciseSyntaxHelpPanelHeaderBar/>
                  }
                >
                  <ExerciseSyntaxHelpPanelView/>
                </TabContentHost>

              </Panel>
            }

            bottomLeftPanel={
              <Panel
                selectedTabIndex={this.props.bottomLeftSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setBottomLeftSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setBotLeftPanelViewSize}
                panelViewSize={this.props.botLeftPanelViewSize}
              >

                <TabContentHost
                  title={getI18n(this.props.langId, 'Custom tests')}
                  icon={'shield'}
                  disabled={this.props.customProject.id <= 0}
                  tabHeaderBar={
                    <CustomTestPanelHeaderBar

                      customTests={this.props.customProject.tests}
                      canUserCreateCustomTests={true}
                      isSomeTestRunning={this.props.isSomeTestRunning}

                      openCreateCustomTestDialog={this.props.openCreateCustomProjectTestDialog}

                      runCustomTests={async (tests: ReadonlyArray<RunnableCustomProjectTestFrontendExclusive>) => {

                        const command: RunTestCommandCustomProjectForBackend = {
                          customProjectId: this.props.customProject.id,
                          plangId: this.props.customProject.solution.pLangId,
                          testIds: tests.map(p => p.id),
                        }

                        const hasAnyTestFailed = await this.props.runCustomProjectTestAsync(command, this.props.langId)

                        return hasAnyTestFailed as any //wrong infered because magic redux binding
                      }}

                      resetCustomTestResults={this.props.resetCustomProjectTestResults}

                      beforeRunningTests={async () => {
                        if (this.props.editorHasUnsavedChanges) {
                          await this.props.saveCustomProjectSolution(this.props.customProject, this.props.langId)
                        }
                      }}

                    />
                  }
                  defaultHelpText={getI18n(this.props.langId,
                                           'If the editor has unsaved changes and you run a test then the files will be auto saved before running the test'
                  )}
                >
                  <CustomTestsPanel
                    areAfterTests={false}
                    customTests={this.props.customProject.tests}
                    canUserEditExercise={true}
                    canUserEditCustomTests={true}
                    isSomeTestRunning={this.props.isSomeTestRunning}
                    selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                    setSelectedConsoleViewTest={this.props.setSelectedConsoleViewTest}
                    openUpdateCustomTestDialog={this.props.openEditCustomProjectTestDialog}

                    runCustomTest={async (test: RunnableCustomProjectTestFrontendExclusive) => {

                      const command: RunTestCommandCustomProjectForBackend = {
                        customProjectId: this.props.customProject.id,
                        plangId: this.props.customProject.solution.pLangId,
                        testIds: [test.id],
                      }

                      const hasAnyTestFailed = await this.props.runCustomProjectTestAsync(command, this.props.langId)

                      return hasAnyTestFailed as any //wrong infered because magic redux binding
                    }}

                    beforeRunningTests={async () => {
                      if (this.props.editorHasUnsavedChanges) {
                        await this.props.saveCustomProjectSolution(this.props.customProject, this.props.langId)
                      }
                    }}
                    deleteCustomTestDoEditor={(test: RunnableCustomProjectTestFrontendExclusive) => {
                      this.props.deleteCustomTestCustomProjectAsync(this.props.customProject.id, test,
                                                                    this.props.customProject.tests
                      )
                    }}

                  />

                </TabContentHost>



                <TabContentHost
                  title={getI18n(this.props.langId, 'Assets')}
                  icon={'file'}
                  tabHeaderBar={
                    <AssetsPanelHeaderBar
                      isEnabled={this.props.customProject.id > 0}
                      assets={this.props.customProjectDescription.assets}
                      setAssets={(assets) => {
                        this.props.setCustomProjectDescriptionAssets(assets)
                      }}
                      openAddAssetsModal={() => {
                        this.props.setIsAddCustomProjectAssetFilesModalDisplayed(true, false) //before loading because this will cancel isBusy
                        this.props.getPossibleAllUserFilesAsync()
                      }}
                    />
                  }
                >
                  <AssetsPanelView assets={this.props.customProjectDescription.assets}
                  />
                </TabContentHost>


              </Panel>
            }

            bottomRightPanel={
              <Panel
                selectedTabIndex={this.props.bottomRightSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setBottomRightSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setBotRightPanelViewSize}
                panelViewSize={this.props.botRightPanelViewSize}
              >
                <TabContentHost
                  title={getI18n(this.props.langId, 'Console output')}
                  icon={'terminal'}
                  tabHeaderBar={
                    <ConsolePanelHeaderBar test={this.props.selectedConsoleViewTest} testTypes={this.props.testTypes}
                    />
                  }
                >
                  <ConsolePanelView test={this.props.selectedConsoleViewTest} protocolItems={null}
                                    testTypes={this.props.testTypes}
                                    displayFilterConsoleItemsOptions={true}
                  />
                </TabContentHost>

              </Panel>
            }


          />
        }

      </div>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(editCustomProjectSite))
